#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <login_cap.h>
#include <pwd.h>
#include <bioapi.h>

#define PAM_SM_AUTH
#define PAM_SM_PASSWORD

#include <security/pam_appl.h>
#include <security/pam_modules.h>
#include <security/pam_mod_misc.h>

#include "bioapi.h"

#define BIR_PATH	"/var/db/bioapi/bir"

/*
 * authentication management
 */
PAM_EXTERN int
pam_sm_authenticate(pam_handle_t *pamh, int flags __unused,
    int argc, const char *argv[])
{
	const char *user, *bsp_uuid;
	char path[FILENAME_MAX];
	int retval, pam_retval = PAM_AUTH_ERR;
	BioAPI_HANDLE *handle;
	uid_t ruid, euid;

	if (argc < 1) {
		PAM_LOG("BiAPI BSP UUID missing");
		return (PAM_SERVICE_ERR);
	}

	euid = geteuid();
	ruid = getuid();

	if (euid != 0)
		return (PAM_PERM_DENIED);

	if (openpam_get_option(pamh, PAM_OPT_AUTH_AS_SELF))
		user = getlogin();
	else {
		retval = pam_get_user(pamh, &user, NULL);	
		if (retval != PAM_SUCCESS)
			return (retval);
	}

	PAM_LOG("Got user: %s", user);
	bsp_uuid = argv[0];
	PAM_LOG("Got BioAPI BSP UUID: %s", bsp_uuid);

	setuid(euid);

	retval = init_bioapi();
	if (retval)
		goto out;

	handle = set_bsp(bsp_uuid);
	if (handle == NULL) {
		PAM_LOG("Faild to initiate BioAPI BSP module");
		goto out;
	}

	snprintf(path, FILENAME_MAX, "%s/%s",
	    (argc > 1) ? argv[1] : BIR_PATH, bsp_uuid);

	retval = verify(handle, user, path); 
	PAM_LOG("Got biometric authentication");
	if (retval == 0)
		pam_retval = PAM_SUCCESS;
	else
		PAM_VERBOSE_ERROR("Biometric authentication refused");

	unload_bsp(handle, bsp_uuid);
	close_bioapi();

out:
	return (pam_retval);
}

PAM_EXTERN int
pam_sm_setcred(pam_handle_t *pamh __unused, int flags __unused,
    int argc __unused, const char *argv[] __unused)
{

    return (PAM_SUCCESS);
}

PAM_EXTERN int
pam_sm_chauthtok(pam_handle_t *pamh, int flags,
    int argc __unused, const char *argv[] __unused)
{

	return (PAM_SUCCESS);
}


PAM_MODULE_ENTRY("pam_bsdbioapi");
